package com.example.todo;

import android.content.Context;
import android.content.ContentProvider;
import android.content.ContentResolver;
import android.content.ContentUris;
import android.content.ContentValues;
import android.content.UriMatcher;

import android.database.Cursor;
import android.database.SQLException;
import android.database.sqlite.SQLiteDatabase;
import android.database.sqlite.SQLiteOpenHelper;
import android.database.sqlite.SQLiteQueryBuilder;

import android.net.Uri;
import android.util.Log;


public class TodoDatabaseProvider extends ContentProvider {

  private static final String TAG = "TodoDBProvider";
  private static final String DB_NAME = "todo.db";
  private static final int DB_VERSION = 1;

  private static final String AUTHORITY = 
      "com.example.todo.tododatabaseprovider";
  private static final String TODO_BASE_PATH = "todo";
  public static final Uri CONTENT_URI = 
      Uri.parse("content://" + AUTHORITY + "/" + TODO_BASE_PATH);
  public static final String CONTENT_ITEM_TYPE = 
      ContentResolver.CURSOR_ITEM_BASE_TYPE + TODO_BASE_PATH;
  public static final String CONTENT_TYPE = 
      ContentResolver.CURSOR_DIR_BASE_TYPE + TODO_BASE_PATH;

  private static final int TODOS = 100;
  private static final int TODO_ID = 101;
  private static final UriMatcher matcher = new UriMatcher(UriMatcher.NO_MATCH);
  static {
    matcher.addURI(AUTHORITY, TODO_BASE_PATH, TODOS);
    matcher.addURI(AUTHORITY, TODO_BASE_PATH + "/#", TODO_ID);
  }

  public static final String ID = "_id";
  public static final String TASK_TABLE = "tasks";
  public static final String COL_TASK_ID = TASK_TABLE + "." + ID;
  public static final String COL_TASK = "task";
  public static final String COL_DUEDATE = "duedate";
  public static final String COL_CREATEDATE = "createdate";
  public static final String COL_CATEGORY_LINK = "category_link";

  public static final String CATEGORY_TABLE = "categories";
  public static final String COL_CATEGORY_ID = CATEGORY_TABLE + "." + ID;
  public static final String COL_CATEGORY= "category";
  public static final String BOTH_TABLES = TASK_TABLE + "," + CATEGORY_TABLE;

  private static TodoDBOpenHelper db;
  private static String tableInUse = BOTH_TABLES;

  @Override
  public boolean onCreate() {
    db = new TodoDBOpenHelper(getContext());
    return true;
  }

  @Override
  public Cursor query(Uri uri, String[] projection, String selection, 
                      String[] selectionArgs, String sortOrder) {
    SQLiteQueryBuilder builder = new SQLiteQueryBuilder();
    builder.setTables(tableInUse);
    int uriType = matcher.match(uri);
    switch (uriType) {
      case TODO_ID:
        builder.appendWhere(COL_TASK_ID + "=" + uri.getLastPathSegment());
        break;
      case TODOS:
        // no filter
        break;
      default:
        throw new IllegalArgumentException("Unknown URI" + uri);
    }
   
    Cursor cursor = builder.query(db.getReadableDatabase(),
                                  projection, selection, selectionArgs, 
                                  null, null, sortOrder);
    cursor.setNotificationUri(getContext().getContentResolver(), uri);
    tableInUse = BOTH_TABLES;
    return cursor; 
  }
  
  public static void setTable(String table) {
    tableInUse = table;
  }
  
  @Override
  public Uri insert(Uri uri, ContentValues values) {
    int uriType = matcher.match(uri);
    if (uriType != TODOS) {
      throw new IllegalArgumentException("Invalid URI for insert");
    }
    SQLiteDatabase sqlDB = db.getWritableDatabase();
    long newID = sqlDB.insert(TASK_TABLE, COL_TASK, values);
    if (newID > 0) {
      Uri newUri = ContentUris.withAppendedId(uri, newID);
      getContext().getContentResolver().notifyChange(uri, null);
      return newUri;
    } else {
    throw new SQLException("Failed to insert row into " + uri 
                           + "result " + newID);
    }
  }

  @Override
  public int update(Uri uri, ContentValues values, String selection,
                    String[] selectionArgs) {
    int uriType = matcher.match(uri);
    if (uriType != TODO_ID) {
      throw new IllegalArgumentException("Invalid URI for update");
    }
    long id = ContentUris.parseId(uri);
    String where = COL_TASK_ID + "='" + id + "'";
    SQLiteDatabase sqlDB = db.getWritableDatabase();
    int result = sqlDB.update(TASK_TABLE, values, where, null);
    // update could in theory update multiple rows; but not in this 
    // app with this code
    if (result != 1) {
      Log.e(TAG, "More than one row updated -- this is probably an error");
    } 
    return result;
  }

  @Override
  public int delete(Uri uri, String selection, String[] selectionArgs) {
    SQLiteDatabase sqlDB = db.getWritableDatabase();
    int result = sqlDB.delete(TASK_TABLE, selection, selectionArgs);
    // delete could in theory delete multiple rows; but not in this 
    // app with this code
    if (result != 1) {
      Log.e(TAG, "More than one row deleted -- this is probably an error");
    }  
    return 1;
  }

  @Override
  public String getType(Uri uri) {
    int uriType = matcher.match(uri);
    switch (uriType) {
      case TODOS:
        return CONTENT_TYPE;
      case TODO_ID:
        return CONTENT_ITEM_TYPE;
      default:
        return null;
    }
  }

  public static SQLiteDatabase getReadableDb() {
    return db.getReadableDatabase();
  }

  /**
   * Creates/opens database
   */
  private static class TodoDBOpenHelper extends SQLiteOpenHelper {

    TodoDBOpenHelper(Context c) {
      super(c, DB_NAME, null, DB_VERSION);
    }

    @Override
    public void onCreate(SQLiteDatabase db) {
      db.execSQL("CREATE TABLE " + TASK_TABLE + " ("
                 + ID + " INTEGER PRIMARY KEY," 
                 + COL_TASK + " TEXT," 
                 + COL_DUEDATE + " DATE,"
                 + COL_CREATEDATE + " INTEGER,"
                 + COL_CATEGORY_LINK + " INTEGER REFERENCES " 
                                     + CATEGORY_TABLE + "(" 
                                     + ID + ")"
                 + ");");
      db.execSQL("CREATE TABLE " + CATEGORY_TABLE + " ("
                 + ID + " INTEGER PRIMARY KEY,"
                 + COL_CATEGORY + " TEXT"
                 + ");");
      db.execSQL("INSERT INTO " + CATEGORY_TABLE + " VALUES(1, 'work');");
      db.execSQL("INSERT INTO " + CATEGORY_TABLE + " VALUES(2, 'personal');");
    }

    @Override
    public void onUpgrade(SQLiteDatabase db, int oldVersion, int newVersion) {
      db.execSQL("DROP TABLE IF EXISTS " + TASK_TABLE);
      db.execSQL("DROP TABLE IF EXISTS " + CATEGORY_TABLE);
      onCreate(db);
    }
  }
}
